function images = pco_adaptor_example_hardwaretrigger(imcount, ...
    triggermode,timeout_s)
% This function records and returns a series of images using a hardware 
% trigger mode of the camera. Therefore the user has to trigger the
% acquisition of each image directly at the trigger input of the camera.
% Always one image per trigger will be recorded, the adaptor doesn't
% control the trigger, it just waits until all images have been recorded.
%
%   [images] = pco_adaptor_example_hardwaretrigger(imcount, ...
%       triggermode, timeout_s)
%
% * Input parameters
%       imcount:        Number of images to acquire
%       triggermode:    Trigger mode to be set 
%                       ('ExternExposureStart' or 'ExternExposureCtrl')
%       timeout_s:      Number of seconds to wait before video input 
%                       timeout occurs
%
% * Output parameters
%       images          Stack with acquired images
%
% - Set property values and trigger mode
% - Start preview, acquire images by hardware trigger signal, stop preview
% - Display images and reset the adaptor
%

%% Set default values for paramters that haven't been set
if(~exist('imcount','var'))
    imcount = 5;
end

if(~exist('triggermode','var'))
    triggermode = 'ExternExposureStart';
end

if(~exist('timeout_s','var'))
    timeout_s = 120;
end

%% Get adaptor name
%Identify required adaptor
if verLessThan('matlab','9.0') %older than R2016a
  error('This adaptor is supported in Matlab R2016a and later versions'); 
else %R2016a and newer
  if(strcmp(computer('arch'),'win64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  elseif(strcmp(computer('arch'),'glnxa64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  else
    error('This platform is not supported.');
  end
end

%% Configure camera and record images
%Create video input object
vid = videoinput(adaptorName, 0);

%Create adaptor source
src = getselectedsource(vid);

%Set timestamp mode
src.TMTimestampMode = 'BinaryAndAscii';

%Set the timeout property of video input object 'vid'
set(vid,'Timeout',timeout_s); 

%Set logging to memory
vid.LoggingMode = 'memory';

%Configure trigger type and mode
triggerconfig(vid, 'hardware', '', triggermode);

%Configure polarity of IO signal at trigger port
src.IO_1SignalPolarity = 'rising';

%Set frames per trigger
vid.FramesPerTrigger = imcount;

%Start preview
preview(vid);

%Start acquisition
start(vid);

%Read out all images and remove them from memory buffer
%Function blocks until the required images have been aquired
images = getdata(vid, imcount);

%Stop preview
stoppreview(vid);

%% Show images
%Show images in Video Viewer app
implay(images);

% Reset adaptor
imaqreset;