function pco_imaqregister(reg)
%PCO_IMAQREGISTER Register/Unregister the PCOCameraAdaptor 
%
% PCO_IMAQREGISTER or PCO_IMAQREGISTER('REGISTER')
% For R2013b - R2015b:
%   - Copies PCOCameraAdaptor_R20XXy_$(Platform).dll from the Adaptors sub folder to the current folder
%   - Copies aditionall required dll's from the $(Platform) (x64 or win32) sub folder to the current folder
%   - Registers the camera adaptor from the current folder
% For R2016a and later:
%   - Copies PCOCameraAdaptor_R20XXy.dll from the Adaptors sub folder to the current folder
%   - Copies aditionall required dll's from the $(Platform) (x64) sub folder to the current folder
%   - Registers the camera adaptor from the current folder
%
% PCO_IMAQREGISTER('UNREGISTER')
% Unregisteres the PCOCameraAdaptor.dll from the registry location
%
% See also IMAQREGISTER.
%
%(c) 2003-2022 Excelitas PCO GmbH * Donaupark 11 * D-93309 Kelheim / Germany 

  if(~exist('reg','var'))
    reg = 'register';
  end

  %get registered adaptors
  adaptors = imaqregister;

  %Identify required adaptor
  if verLessThan('matlab','9.0') %older than R2016a
    error('This adaptor is supported in Matlab R2016a and later versions'); 
  else %R2016a and newer
    if(strcmp(computer('arch'),'win64'))
      adaptorName = ['PCOCameraAdaptor_R' version('-release') '.dll'];
    elseif(strcmp(computer('arch'),'glnxa64'))
      adaptorName = ['PCOCameraAdaptor_R' version('-release') '.so'];
    else
      error('This platform is not supported.');
    end
  end

  if (strcmp(reg,'register'))
    %Copy Adaptor
    if(exist(['./Adaptors/' adaptorName],'file'))
      if(exist(['./' adaptorName],'file'))
        delete(['./' adaptorName]);
      end
      copyfile(['./Adaptors/' adaptorName],'./');
    end
    
    %Copy additionall dll files
    if(strcmp(computer('arch'),'win32'))
      error('This adaptor is not supported for 32-Bit Windows platforms');      
    elseif(strcmp(computer('arch'),'win64'))
      %SC2_Cam.dll
      if(exist('../runtime/win_x64/bin/sc2_cam.dll','file'))
        if(exist('./sc2_cam.dll','file'))
          delete('./sc2_cam.dll');
        end
        copyfile('../runtime/win_x64/bin/sc2_cam.dll','./');
      end
      %sc2_cl_me4.dll
      if(exist('../runtime/win_x64/bin/sc2_cl_me4.dll','file'))
        if(exist('./sc2_cl_me4.dll','file'))
          delete('./sc2_cl_me4.dll');
        end
        copyfile('../runtime/win_x64/bin/sc2_cl_me4.dll','./');
      end
      %sc2_clhs.dll
      if(exist('../runtime/win_x64/bin/sc2_clhs.dll','file'))
        if(exist('./sc2_clhs.dll','file'))
          delete('./sc2_clhs.dll');
        end
        copyfile('../runtime/win_x64/bin/sc2_clhs.dll','./');
      end
      %sc2_cl_nat.dll
      if(exist('../runtime/win_x64/bin/sc2_cl_nat.dll','file'))
        if(exist('./sc2_cl_nat.dll','file'))
          delete('./sc2_cl_nat.dll');
        end
        copyfile('../runtime/win_x64/bin/sc2_cl_nat.dll','./');
      end
      %sc2_cl_mtx.dll
      if(exist('../runtime/win_x64/bin/sc2_cl_mtx.dll','file'))
        if(exist('./sc2_cl_mtx.dll','file'))
          delete('./sc2_cl_mtx.dll');
        end
        copyfile('../runtime/win_x64/bin/sc2_cl_mtx.dll','./');
      end
      %sc2_gige.dll
      if(exist('../runtime/bin64/sc2_gige.dll','file'))
        if(exist('./sc2_gige.dll','file'))
          delete('./sc2_gige.dll');
        end
        copyfile('../runtime/bin64/sc2_gige.dll','./');
      end
      %sc2_genicam.dll
      if(exist('../runtime/win_x64/bin/sc2_genicam.dll','file'))
        if(exist('./sc2_genicam.dll','file'))
          delete('./sc2_genicam.dll');
        end
        copyfile('../runtime/win_x64/bin/sc2_genicam.dll','./');
      end
      %sc2_gige.dll
      if(exist('../runtime/win_x64/bin/sc2_gige.dll','file'))
        if(exist('./sc2_gige.dll','file'))
          delete('./sc2_gige.dll');
        end
        copyfile('../runtime/win_x64/bin/sc2_gige.dll','./');
      end
      if(exist('../runtime/win_x64/bin/genicam','dir'))
        if(exist('./genicam','dir'))
          rmdir('./genicam', 's')
        end
        copyfile('../runtime/win_x64/bin/genicam','./genicam');
      end
    %all linux binaries
    elseif(strcmp(computer('arch'),'glnxa64'))
      delete('./lib*'); %also symlinks
      copyfile('../runtime/lnx_amd64/lib/lib*.so.*.*.*', './');
      % create symlinks
      libs = dir('./lib*.so.*.*.*');
      for l=1:numel(libs)
        lib_full = libs(l).name;
        folder = libs(l).folder;
        ss = strsplit(lib_full, '.');
        lib_major = [ss{1}, '.' ss{2} '.' ss{3}];
        lib_so = [ss{1}, '.', ss{2}];
        %                folder '/' 
        system(['ln -sf ' lib_full ' ' lib_major]);
        system(['ln -sf ' lib_major ' ' lib_so]);
      end

    else
      error('This platform is not supported.');
    end

    %check if an old adaptor is already registered and unregister if necessary
    if (isempty(adaptors)== 0)
      for i= 1:length(adaptors)
        [~, name, ext] = fileparts(char(adaptors(i)));
        if(strcmp([name ext], 'PCOCameraAdaptor.dll'))
          imaqregister(char(adaptors(i)), 'unregister');
          warning('An old adaptor version was already registered. This adaptor is unregistered now.');
        elseif (strcmp([name ext], adaptorName))
          imaqregister(char(adaptors(i)), 'unregister');
          warning('An adaptor with the same name was already registered. This adaptor is unregistered now.');
        end
      end
    end

    %Register adaptor
    if(strcmp(computer('arch'),'glnxa64'))
      path = [pwd '/' adaptorName];
    else
      path = [pwd '\' adaptorName];
    end

    imaqregister(path, 'register');
    
  elseif (strcmp(reg,'unregister'))
    if isempty(adaptors)
      disp('No adaptors registered. There is nothing to unregister.');
    else
      %check if adaptor is registered at all, if so do unregister
      for i= 1:length(adaptors)
        if(isempty(strfind(adaptors(i), adaptorName)) == 0)
          imaqregister(char(adaptors(i)), 'unregister');
          break;
        elseif (i==length(adaptors))
          disp([adaptorName ' was not found. There is nothing to unregister.']);
        end
      end
    end
    
  else
    warning('Wrong input string. Use register or unregister as valid inputs.');
  end

end
