function [phasor normInt] = flim_numeric_harmonic_analysis_phasor(...
    imgStack, nBits, nPhases, harmonic)

%
% function [phasor normInt] = flim_numeric_harmonic_analysis_phasor(...
%     imgStack, nBits, nPhases, harmonic)
%
% This function computes the complex phasor (magnitude: modulation index,
% angle: phase) and normalized intensity out of an image stack containing
% intensity images recorded at different equidistant phases for a specific
% harmonic component.
% The phases of the image stack should be concatenated in an ascending
% order.
%
% Input parameters:
% imgStack: 3D image stack containing phase information along 3rd dimension
% nBits:    Bit resolution of imgStack raw intensity values
% nPhases:  Number of phases
% harmonic: Harmonic component to be computed (1 for fundamental)
%
% Return values:
% phasor:  2D complex phasor image
% normInt: 2D normalized intensity image
%

    % initialize complex sum
    z = complex(zeros(size(imgStack(:, :, 1))));

    % compute complex unity roots
    w = exp(-i * 2 * pi * harmonic * [0:nPhases - 1] / nPhases);

    % compute complex sum
    for iPhase = 1:nPhases
        z = z + w(iPhase) .* double(imgStack(:, :, iPhase));
    end

    % compute complex phasor, convert magnitude of z into modulation index
    phasor = (2 ./ (mean(imgStack, 3) * nPhases + realmin)) .* z;

    b = mean(imgStack, 3); % constant component
    normInt = b / 2^nBits;

end
